; ------------------------------------------------------------------------------------
; EasyCluster Include for PB
; Author: Alexander Aigner
; Easy Cluster DLL (c) Alexander Aigner
; ------------------------------------------------------------------------------------

EnableExplicit

Structure ECluster_Mem
  *Mem
  MemSize.i
EndStructure

; ------------------------------------------------------------------------------------
; Mgliche Rckgabewerte von ECluster_Master_Event_Type()
; ------------------------------------------------------------------------------------

Enumeration 1
  ; Job Events (01 - 9F)
  #Event_Job_Finished = $01 ; Job beendet
  #Event_Job_InProgress = $02 ; Job in Arbeit
  #Event_Job_Received = $03 ; Job wurde empfangen
  #Event_Job_Message = $04 ; Meldung vom Job
  #Event_Job_Limit = $05 ; Job Limit erreicht, wird nicht ausgefhrt
  #Event_Job_Unknown = $06 ; Job unbekannt
  #Event_Job_Denied = $07 ; Job wurde verweigert
  #Event_Job_Invalid = $08 ; Ungltige Job - Id
  #Event_Job_Error = $9F ; Job Fehler (wegen Buffer-berlauf)
  
  ; Slave Events (A0 - EF)
  #Event_Slave_Quit = $A0 ; Slave beendet
  #Event_Slave_Paused = $A1 ; Slave Pausiert
  #Event_Slave_Resumed = $A2 ; Slave macht weiter
  #Event_Slave_Kick = $A3; Master wurde gekickt
  #Event_Slave_UnknownCommand = $A4; Unbekanntes Kommando
  
  ; Sonstiges (F0 - FF)
  #Event_Master_BufferUnderrun = $FE; Buffer Unterlauf
  #Event_Slave_BufferUnderrun = $FF; Buffer Unterlauf
EndEnumeration

; ------------------------------------------------------------------------------------
; Mgliche Rckgabewerte von ECluster_Slave_Event_Type()
; ------------------------------------------------------------------------------------

Enumeration 1
  ; Master Events (01 - EF)
  #Event_Master_Disconnect = $01; Master beendet Verbindung
  #Event_Master_Connect = $02 ; Master Verbunden
  #Event_Master_GetJobInfos = $03 ; Master fordert Jobliste an
  #Event_Master_DoJob = $04 ; Master fordert Job an
  #Event_Master_CountJobs = $05 ; Master zhlt verbleibende Jobs im Queue
  #Event_Master_CountWorkers = $06 ; Master zhlt Worker am Slave
  #Event_Master_UnknownCommand = $07 ; Unbekanntes Kommando
  #Event_Master_Ping = $08 ; Master Pingt Client
  
  ; Sonstiges (F0 - FF)
  #Event_Slave_BufferUnderrun = #Event_Slave_BufferUnderrun ; Buffer Unterlauf
EndEnumeration

; ------------------------------------------------------------------------------------
; Priorittsklassen fr Jobs
; ------------------------------------------------------------------------------------

Enumeration 0 ; Priority - Klassen
  #Job_LowPriority
  #Job_NormalPriority
  #Job_HighPriority
  #Job_HighestPriority
  #Job_RealTimePriority
EndEnumeration

; ------------------------------------------------------------------------------------
; Von ECluster aufgerufenen Proceduren mssen folgendes Format haben
;
; ECluster_Slave: Slave Handle das die Procedure aufruft
; Master Number: Master Nummer fr die der Job ausgefhrt wird
; JobNumber: Aktuelle Job - Nummer
; ParameterStack: Stack mit den bergabe - Parametern oder 0 wenn keine Parameter bergeben wurden
;
; Mgliche Rckgabewerte:
; 0 oder die ECluster_Return_*() Funktionen benutzen
; ------------------------------------------------------------------------------------

Prototype ECluster_Job(ECluster_Slave, MasterNumber, JobNumber, ParameterStack)

PrototypeC ECluster_Job_C(ECluster_Slave, MasterNumber, JobNumber, ParameterStack)

; ------------------------------------------------------------------------------------
; Callback fr ECluster_Master_IterateSlaves()
; ECluster_Master: Master Handle
; SlaveNumber: aktuelle Slave Nummer
; UserData: angegebene UserData
;
; Rckgabe: <> #False: Iteration fortsetzen, #False: Iteration abbrecheb
; ------------------------------------------------------------------------------------

Prototype ECluster_Master_SlaveCB(ECluster_Master, SlaveNumber, UserData)

; ------------------------------------------------------------------------------------
; Callback fr ECluster_Slave_IterateMasters()
;
; ECluster_Slave: Slave Handle
; MasterNumber: aktuelle Master Nummer
; UserData: angegebene UserData
;
; Rckgabe: <> #False: Iteration fortsetzen, #False: Iteration abbrecheb
; ------------------------------------------------------------------------------------

Prototype ECluster_Slave_MasterCB(ECluster_Slave, MasterNumber, UserData)

; ------------------------------------------------------------------------------------
; Callback fr Master - Events
;
; ECluster_Master: Master Handle
; SlaveNumber: aktuelle Slave Nummer
; Job: Job Nummer
; EventType: EventTyp (wie ECluster_Master_Event_Type())
; *Mem: Speicheradresse (wenn EventType = #Event_Job_Finished)
; MemSize: Speichergre (wenn EventType = #Event_Job_Finished)
; UserData: angegebene UserData
;
; Rckgabe: <> #False: Iteration fortsetzen, #False: Iteration abbrecheb
; ------------------------------------------------------------------------------------

Prototype ECluster_Master_EventCB(ECluster_Master, SlaveNumber, Job, EventType, *Mem, MemSize, UserData)

; ------------------------------------------------------------------------------------
; Callback fr Slave - Events
;
; ECluster_Slave: Slave Handle
; MasterNumber: aktuelle Master Nummer
; Job$: Job Name
; EventType: EventTyp (wie ECluster_Slave_Event_Type())
; UserData: angegebene UserData
;
; Rckgabe: <> #False: Iteration fortsetzen, #False: Iteration abbrechen
; ------------------------------------------------------------------------------------

Prototype ECluster_Slave_EventCB(ECluster_Slave, MasterNumber, Job$, EventType, UserData)

; ------------------------------------------------------------------------------------
; Callback fr Zugriffscheck bei Jobs
;
; ECluster_Slave: Slave Handle
; MasterNumber: aktuelle Master Nummer
; *Job: Job Name
;
; Rckgabe: <> #False: Job ausfhren, #False: Job verweigern
; ------------------------------------------------------------------------------------

Prototype ECluster_Slave_JobAccessCheck(ECluster_Slave, MasterNumber, Job$)

; ------------------------------------------------------------------------------------
;- Slave Routinen
;
; Routinen zur Steuerung eines Slave
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Initialisiert einen neuen Slave
;
; Port: Port an dem der Slave luft
; NumberOfThreads: Anzahl der Worker - Threads des Slave
;
; Rckgabe:
; Slave - Handle wenn Ok, #False wenn Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Init(Port, NumberOfThreads = 1)

; ------------------------------------------------------------------------------------
; Beendet einen Slave
;
; ECluster_Slave: Slave Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Deinit(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Pausiert die Abarbeitung der JobQueue
;
; ECluster_Slave: Slave Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Pause(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Setzt die Abarbeitung der JobQueue fort
;
; ECluster_Slave: Slave Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Resume(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Zhlt die Anzahl der Masters am Slave
;
; ECluster_Slave: Slave Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_CountMasters(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Iteriert die Masters des Slave und sendet sie an ein CallBack
;
; ECluster_Slave: Slave Handle
; *CallBack: Pointer auf ECluster_Slave_MasterCB Procedure
; UserData: Individuelle Daten
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_IterateMasters(ECluster_Slave, *CallBack.ECluster_Slave_MasterCB, UserData)

; ------------------------------------------------------------------------------------
; Kickt einen Master vom Slave
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_KickMaster(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Setzt die das Job - Limit fr einen/alle Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer (0 = Alle Master)
; Limit: Job Limit (0 : kein Limit, <=-1 : Keine Jobs)
;
; Rckgabe: altes Job - Limit
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetMasterJobLimit(ECluster_Slave, Master, Limit)

; ------------------------------------------------------------------------------------
; Setzt Callback fr den Zugangscheck fr einen/alle Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer (0 = Alle Master)
; *CallBack: CallBack (siehe ECluster_Slave_JobAccessCheck)
;
; Rckgabe: altes Job - Limit
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetMasterJobAccessCheck(ECluster_Slave, Master, *Callback)

; ------------------------------------------------------------------------------------
; Ermittelt die Anzahl austehender Jobs fr einen Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
;
; Rckgabe: aktuelle Job-Anzahl
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_CountMasterJobs(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Ermittelt die IP des Masters (zur Verarbeitung mit IPString())
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_GetMasterIP(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Ermittelt den Port des Masters
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_GetMasterPort(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Holt die gespeicherten Xtra-Daten des Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
;
; Rckgabe:
; Xtra - Data des Master
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_GetMasterXtraData(ECluster_Slave, Master)

; ------------------------------------------------------------------------------------
; Setzt die Xtra-Daten des Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
; XtraData: XtraDaten
;
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetMasterXtraData(ECluster_Slave, Master, XtraData)

; ------------------------------------------------------------------------------------
; Ermittelt die Anzahl der ausstehenden Jobs
;
; ECluster_Slave: Slave Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_CountJobs(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Setzt die EmpfangsBlock - Puffergre
;
; ECluster_Slave: Slave Handle
; Size: neue EmpfangsBlock - Puffergre in Bytes > 0 (Default: 64 Kb)
;
; Rckgabe:
; Alte Puffergre
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetReceiveBlockSize(ECluster_Slave, Size = 64*1024)

; ------------------------------------------------------------------------------------
; Setzt die Einstellungen fr die Automatische Kompression
;
; ECluster_Slave: Slave Handle
; Limit: Limit ab dem Komprimiert werden soll (0: deaktiviert, <0: alles komrimieren)
; Level: Kompressions-Level (1-9, 0 = deaktiviert)
;
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetCompressionOptions(ECluster_Slave, Limit, Level)

; ------------------------------------------------------------------------------------
; Setzt die Einstellungen fr ein globales Callback
;
; ECluster_Slave: Slave Handle
; *CallBack: CallBack (siehe ECluster_Slave_EventCB)
; UserData: UserData fr Callback
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_SetCallback(ECluster_Slave, *CallBack, UserData)

; ------------------------------------------------------------------------------------
; Aktiviert den Thread Empfangsmodus des Slave
;
; ECluster_Slave: Slave Handle
; State: <> 0: Aktiviert, 0: Deaktiviert
;
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_ThreadReceive(ECluster_Slave, State)

; ------------------------------------------------------------------------------------
; Fgt einen verfgbaren Job zum Slave hinzu
;
; ECluster_Slave: Slave Handle
; JobName$: Name des Job
; JobDescription$: Job Beschreibung
; JobProcedure: Pointer auf eine ECluster_Job() Procedure
; cdecl: Verwendet die cdecl - Aufrufkonvention fr den Job (Standard: stdcall)
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_AddJob(ECluster_Slave, JobName$, JobDescription$, JobProcedure.ECluster_Job, cdecl = 0)

; ------------------------------------------------------------------------------------
; Schaltet Events auf dem Slave ein/aus
;
; ECluster_Slave: Slave Handle
; Enable: <> 0: Events ein, 0: Events aus
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_LogEvents(ECluster_Slave, Enable)

; ------------------------------------------------------------------------------------
; Prft ob Events vorhanden sind
;
; ECluster_Slave: Slave Handle
;
; Rckgabe:
; Event-Anzahl wenn vorhanden, sonst #False
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Event(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Typ des aktuellen Events
;
; ECluster_Slave: Slave Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Event_Type(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Master des aktuellen Events
;
; ECluster_Slave: Slave Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Event_Master(ECluster_Slave)

; ------------------------------------------------------------------------------------
; JobName des aktuellen Events
;
; ECluster_Slave: Slave Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Event_Job(ECluster_Slave)

; ------------------------------------------------------------------------------------
; Sendet Benutzerdefinierte Daten an den Master
;
; ECluster_Slave: Slave Handle
; Master: Master Nummer
; JobNumber: Job Nummer
; *Mem: Speicherblock
; MemSize: Speichergre
;
; ------------------------------------------------------------------------------------

Prototype _ECluster_Slave_Job_SendMessage(ECluster_Slave, Master, JobNumber, *Mem, MemSize)

; ------------------------------------------------------------------------------------
;- Master Routinen
;
; Routinen zur Steuerung eines Master
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Initialisiert einen neuen Master
;
; Rckgabe:
; Master - Handle wenn Ok, #False wenn Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Init()

; ------------------------------------------------------------------------------------
; Beendet einen Master
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_DeInit(ECluster_Master)

; ------------------------------------------------------------------------------------
; Setzt die EmpfangsBlock - Puffergre
;
; ECluster_Master: Slave Handle
; Size: neue EmpfangsBlock - Puffergre in Bytes > 0 (Default: 64 Kb)
;
; Rckgabe:
; Alte Puffergre
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_SetReceiveBlockSize(ECluster_Master, Size = 64*1024)

; ------------------------------------------------------------------------------------
; Setzt die Einstellungen fr die Automatische Kompression
;
; ECluster_Master: Master Handle
; Limit: Limit ab dem Komprimiert werden soll (0: deaktiviert, <0: alles komrimieren)
; Level: Kompressions-Level (1-9, 0 = deaktiviert)
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_SetCompressionOptions(ECluster_Master, Limit, Level)

; ------------------------------------------------------------------------------------
; Setzt die Einstellungen fr ein globales Callback
;
; ECluster_Master: Master Handle
; *CallBack: CallBack (siehe ECluster_Master_EventCB)
; UserData: UserData fr Callback
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_SetCallback(ECluster_Master, *CallBack, UserData)

; ------------------------------------------------------------------------------------
; Aktiviert den Thread Empfangsmodus des Slave
;
; ECluster_Master: Master Handle
; State: <> 0: Aktiviert, 0: Deaktiviert
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_ThreadReceive(ECluster_Master, State)

; ------------------------------------------------------------------------------------
; Fgt einen Slave zum Master hinzu
;
; ECluster_Master: Master Handle
; Address$: Netzwerkaddresse des Slave
; Port: Portnummer des Slave
;
; Rckgabe:
; Slave - Nummer wenn Ok, #False wenn Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_AddSlave(ECluster_Master, Address$, Port.l)

; ------------------------------------------------------------------------------------
; Entfernt einen Slave vom Master
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_RemoveSlave(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Zhlt die Anzahl der Slaves am Master
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_CountSlaves(ECluster_Master)

; ------------------------------------------------------------------------------------
; Iteriert die Slaves des Masters und sendet sie an ein CallBack
;
; ECluster_Master: Master Handle
; *CallBack: Pointer auf ECluster_Master_SlaveCB Procedure
; UserData: Individuelle Daten
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_IterateSlaves(ECluster_Master, *CallBack.ECluster_Master_SlaveCB, UserData)

; ------------------------------------------------------------------------------------
; Ermittelt die IP des Slave (zur Verarbeitung mit IPString())
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_GetSlaveIP(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Holt die gespeicherten Xtra-Daten des Slave
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe:
; Xtra - Data des Slave
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_GetSlaveXtraData(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Setzt die Xtra-Daten des Slave
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
; XtraData: XtraDaten
;
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_SetSlaveXtraData(ECluster_Master, Slave, XtraData)

; ------------------------------------------------------------------------------------
; Pingt den Slave
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe:
; Ping Zeit in Ms
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_PingSlave(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Frag eine Job Liste vom Slave an
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
; ForceRefresh: Liste neu anfordern wenn schon geladen
;
; Rckgabe:
; Job Liste incl. Beschreibungen
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_GetSlaveJobInfos(ECluster_Master, Slave, ForceRefresh = 0)

; ------------------------------------------------------------------------------------
; Frag die Anzahl von anstehenden Jobs am Slave ab
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe:
; Anzahl Jobs im JobQueue des Slave
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_CountSlaveJobs(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Frag die Anzahl der Worker am Slave ab
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
;
; Rckgabe:
; Anzahl Jobs im JobQueue des Slave
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_CountSlaveWorkers(ECluster_Master, Slave)

; ------------------------------------------------------------------------------------
; Fhrt einen Job am Slave aus
;
; ECluster_Master: Master Handle
; Slave: Slave Nummer
; ParameterStack: ParameterStack (ECluster_Parameter_New())
; *CallBack: CallBack fr Optinales EventCB (siehe ECluster_Master_EventCB)
; UserData: UserData fr EventCB
; Priority: Job Prioritt
;
; Rckgabe:
; Job Id wenn Ok, #False wenn Fehler
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_DoJob(ECluster_Master, JobName$, Slave, ParameterStack, *CallBack = 0, UserData = 0, Priority = #Job_NormalPriority)

; ------------------------------------------------------------------------------------
; Wartet bis ein Event eintritt
;
; ECluster_Master: Master Handle
; TimeOut: Optinales Timeout
;
; Rckgabe:
; 0>: JobId , -1: Fehler beim Senden
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_WaitEvent(ECluster_Master, TimeOut = 0)

; ------------------------------------------------------------------------------------
; Prft ob Events vorhanden sind
;
; ECluster_Master: Master Handle
;
; Rckgabe:
; Event-Anzahl wenn vorhanden, sonst #False
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event(ECluster_Master)

; ------------------------------------------------------------------------------------
; Typ des aktuellen Events
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_Type(ECluster_Master)

; ------------------------------------------------------------------------------------
; Slave des aktuellen Events
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_Slave(ECluster_Master)

; ------------------------------------------------------------------------------------
; JobId des aktuellen Events
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_JobId(ECluster_Master)

; ------------------------------------------------------------------------------------
; Folgende Routinen holen den Rckgabewert eines Jobs
; ACHTUNG: Jeder Wert kann nur 1* gelesen werden
;
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs
;
; ECluster_Master: Master Handle
;
; Rckgabe:
; Pointer auf eine ECluster_Mem Structur
;
; ACHTUNG: Wenn sie den Speicher mit diesem Befehl selbst holen mssen sie den Speicher
; und die Structur selbst freigeben.
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_JobMem(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher - Gre des aktuellen EventJobs
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_JobMemSize(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Long
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype.l _ECluster_Master_Event_JobLong(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Integer
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype.i _ECluster_Master_Event_JobInteger(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Float
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype.f _ECluster_Master_Event_JobFloat(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Double
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype.d _ECluster_Master_Event_JobDouble(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als Quad
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype.q _ECluster_Master_Event_JobQuad(ECluster_Master)

; ------------------------------------------------------------------------------------
; RckgabeSpeicher des aktuellen EventJobs als String
;
; ECluster_Master: Master Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Master_Event_JobString(ECluster_Master)

; ------------------------------------------------------------------------------------
;- Parameter Routinen
;
; Routinen um bergabeparameter schreiben und zu lesen
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Neuen ParameterStack anlegen
;
; Rckgabe:
; Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_New()

; ------------------------------------------------------------------------------------
; Parameter auf Stack zhlen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_Count(ParameterStack)

; ------------------------------------------------------------------------------------
; Speicher zum ParameterStack hinzufgen
;
; ParameterStack: Parameter - Handle
; *Mem : Speicheraddresse
; MemSize: Speichergre
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_AddMem(ParameterStack, *Mem, MemSize)

; ------------------------------------------------------------------------------------
; Long zum ParameterStack hinzufgen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_AddLong(ParameterStack, Long.l)

; ------------------------------------------------------------------------------------
; Integer zum ParameterStack hinzufgen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_AddInteger(ParameterStack, Integer.i)

; ------------------------------------------------------------------------------------
; Float zum ParameterStack hinzufgen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_AddFloat(ParameterStack, Float.f)

; ------------------------------------------------------------------------------------
; Double zum ParameterStack hinzufgen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_AddDouble(ParameterStack, Double.d)

; ------------------------------------------------------------------------------------
; Quad zum ParameterStack hinzufgen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_AddQuad(ParameterStack, Quad.q)

; ------------------------------------------------------------------------------------
; String zum ParameterStack hinzufgen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_AddString(ParameterStack, String.s)

; ------------------------------------------------------------------------------------
; Memory vom ParameterStack holen
;
; ParameterStack: Parameter - Handle
;
; Rckgabe:
; Pointer auf ECluster_Mem- Structure
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_GetMem(ParameterStack)

; ------------------------------------------------------------------------------------
; Long vom ParameterStack holen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype.l _ECluster_Parameter_GetLong(ParameterStack)

; ------------------------------------------------------------------------------------
; Integer vom ParameterStack holen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype.i _ECluster_Parameter_GetInteger(ParameterStack)

; ------------------------------------------------------------------------------------
; Float vom ParameterStack holen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype.f _ECluster_Parameter_GetFloat(ParameterStack)

; ------------------------------------------------------------------------------------
; Double vom ParameterStack holen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype.d _ECluster_Parameter_GetDouble(ParameterStack)

; ------------------------------------------------------------------------------------
; Quad vom ParameterStack holen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype.q _ECluster_Parameter_GetQuad(ParameterStack)

; ------------------------------------------------------------------------------------
; String vom ParameterStack holen
;
; ParameterStack: Parameter - Handle
; ------------------------------------------------------------------------------------

Prototype _ECluster_Parameter_GetString(ParameterStack)

; ------------------------------------------------------------------------------------
;- Rckgabe Routinen
;
; Routinen zum zurckgeben von Daten aus den Servive - Proceuduren
; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Speicher aus ECluster_Job() - Procedure zurckgeben
;
; *Mem: Speicheraddresse
; MemSize: Speichergre
; ------------------------------------------------------------------------------------

Prototype _ECluster_Return_Memory(*Mem, MemSize)

; ------------------------------------------------------------------------------------
; Long aus ECluster_Job() - Procedure zurckgeben
; ------------------------------------------------------------------------------------

Prototype _ECluster_Return_Long(Long.l)

; ------------------------------------------------------------------------------------
; Integer aus ECluster_Job() - Procedure zurckgeben
; ------------------------------------------------------------------------------------

Prototype _ECluster_Return_Integer(Integer.i)

; ------------------------------------------------------------------------------------
; Float aus ECluster_Job() - Procedure zurckgeben
; ------------------------------------------------------------------------------------

Prototype _ECluster_Return_Float(Float.f)

; ------------------------------------------------------------------------------------
; Double aus ECluster_Job() - Procedure zurckgeben
; ------------------------------------------------------------------------------------

Prototype _ECluster_Return_Double(Double.d)

; ------------------------------------------------------------------------------------
; Quad aus ECluster_Job() - Procedure zurckgeben
; ------------------------------------------------------------------------------------

Prototype _ECluster_Return_Quad(Quad.q)

; ------------------------------------------------------------------------------------
; String aus ECluster_Job() - Procedure zurckgeben
; ------------------------------------------------------------------------------------

Prototype _ECluster_Return_String(String.s)

; ------------------------------------------------------------------------------------

; ------------------------------------------------------------------------------------
; Memory - Dummy Funktionen da sonst die Speicherverwaltung von Windows beleidigt ist
; ------------------------------------------------------------------------------------

Prototype _ECluster_AllocateMemory(MemSize)
Prototype _ECluster_FreeMemory(Mem)

Global ECluster_Slave_Init._ECluster_Slave_Init
Global ECluster_Slave_DeInit._ECluster_Slave_DeInit
Global ECluster_Slave_Pause._ECluster_Slave_Pause
Global ECluster_Slave_Resume._ECluster_Slave_Pause
Global ECluster_Slave_CountMasters._ECluster_Slave_CountMasters
Global ECluster_Slave_IterateMasters._ECluster_Slave_IterateMasters
Global ECluster_Slave_KickMaster._ECluster_Slave_KickMaster
Global ECluster_Slave_SetMasterJobLimit._ECluster_Slave_SetMasterJobLimit
Global ECluster_Slave_CountMasterJobs._ECluster_Slave_CountMasterJobs
Global ECluster_Slave_GetMasterIP._ECluster_Slave_GetMasterIP
Global ECluster_Slave_GetMasterPort._ECluster_Slave_GetMasterPort
Global ECluster_Slave_GetMasterXtraData._ECluster_Slave_GetMasterXtraData
Global ECluster_Slave_SetMasterXtraData._ECluster_Slave_SetMasterXtraData
Global ECluster_Slave_SetMasterJobAccessCheck._ECluster_Slave_SetMasterJobAccessCheck
Global ECluster_Slave_CountJobs._ECluster_Slave_CountJobs
Global ECluster_Slave_SetReceiveBlockSize._ECluster_Slave_SetReceiveBlockSize
Global ECluster_Slave_SetCompressionOptions._ECluster_Slave_SetCompressionOptions
Global ECluster_Slave_SetCallback._ECluster_Slave_SetCallback
Global ECluster_Slave_ThreadReceive._ECluster_Slave_ThreadReceive
Global ECluster_Slave_AddJob._ECluster_Slave_AddJob
Global ECluster_Slave_LogEvents._ECluster_Slave_LogEvents
Global ECluster_Slave_Event._ECluster_Slave_Event
Global ECluster_Slave_Event_Type._ECluster_Slave_Event_Type
Global ECluster_Slave_Event_Master._ECluster_Slave_Event_Master
Global _ECluster_Slave_Event_Job._ECluster_Slave_Event_Job
Global ECluster_Slave_Job_SendMessage._ECluster_Slave_Job_SendMessage

Global ECluster_Master_Init._ECluster_Master_Init
Global ECluster_Master_DeInit._ECluster_Master_DeInit
Global ECluster_Master_SetReceiveBlockSize._ECluster_Master_SetReceiveBlockSize
Global ECluster_Master_SetCompressionOptions._ECluster_Master_SetCompressionOptions
Global ECluster_Master_SetCallback._ECluster_Master_SetCallback
Global ECluster_Master_ThreadReceive._ECluster_Master_ThreadReceive
Global ECluster_Master_AddSlave._ECluster_Master_AddSlave
Global ECluster_Master_RemoveSlave._ECluster_Master_RemoveSlave
Global ECluster_Master_CountSlaves._ECluster_Master_CountSlaves
Global ECluster_Master_IterateSlaves._ECluster_Master_IterateSlaves
Global ECluster_Master_GetSlaveIP._ECluster_Master_GetSlaveIP
Global ECluster_Master_GetSlaveXtraData._ECluster_Master_GetSlaveXtraData
Global ECluster_Master_SetSlaveXtraData._ECluster_Master_SetSlaveXtraData
Global ECluster_Master_PingSlave._ECluster_Master_PingSlave
Global _ECluster_Master_GetSlaveJobInfos._ECluster_Master_GetSlaveJobInfos
Global ECluster_Master_CountSlaveJobs._ECluster_Master_CountSlaveJobs
Global ECluster_Master_CountSlaveWorkers._ECluster_Master_CountSlaveWorkers
Global ECluster_Master_DoJob._ECluster_Master_DoJob

Global ECluster_Master_WaitEvent._ECluster_Master_WaitEvent
Global ECluster_Master_Event._ECluster_Master_Event
Global ECluster_Master_Event_Type._ECluster_Master_Event_Type
Global ECluster_Master_Event_Slave._ECluster_Master_Event_Slave
Global ECluster_Master_Event_JobId._ECluster_Master_Event_JobId
Global ECluster_Master_Event_JobMem._ECluster_Master_Event_JobMem
Global ECluster_Master_Event_JobMemSize._ECluster_Master_Event_JobMemSize
Global ECluster_Master_Event_JobLong._ECluster_Master_Event_JobLong
Global ECluster_Master_Event_JobInteger._ECluster_Master_Event_JobInteger
Global ECluster_Master_Event_JobFloat._ECluster_Master_Event_JobFloat
Global ECluster_Master_Event_JobDouble._ECluster_Master_Event_JobDouble
Global ECluster_Master_Event_JobQuad._ECluster_Master_Event_JobQuad
Global _ECluster_Master_Event_JobString._ECluster_Master_Event_JobString

Global ECluster_Parameter_New._ECluster_Parameter_New
Global ECluster_Parameter_AddMem._ECluster_Parameter_AddMem
Global ECluster_Parameter_AddLong._ECluster_Parameter_AddLong
Global ECluster_Parameter_AddInteger._ECluster_Parameter_AddInteger
Global ECluster_Parameter_AddFloat._ECluster_Parameter_AddFloat
Global ECluster_Parameter_AddDouble._ECluster_Parameter_AddDouble
Global ECluster_Parameter_AddQuad._ECluster_Parameter_AddQuad
Global ECluster_Parameter_AddString._ECluster_Parameter_AddString
Global ECluster_Parameter_GetMem._ECluster_Parameter_GetMem
Global ECluster_Parameter_GetLong._ECluster_Parameter_GetLong
Global ECluster_Parameter_GetInteger._ECluster_Parameter_GetInteger
Global ECluster_Parameter_GetFloat._ECluster_Parameter_GetFloat
Global ECluster_Parameter_GetDouble._ECluster_Parameter_GetDouble
Global ECluster_Parameter_GetQuad._ECluster_Parameter_GetQuad
Global _ECluster_Parameter_GetString._ECluster_Parameter_GetString

Global ECluster_Return_Memory._ECluster_Return_Memory
Global ECluster_Return_Long._ECluster_Return_Long
Global ECluster_Return_Integer._ECluster_Return_Integer
Global ECluster_Return_Float._ECluster_Return_Float
Global ECluster_Return_Double._ECluster_Return_Double
Global ECluster_Return_Quad._ECluster_Return_Quad
Global ECluster_Return_String._ECluster_Return_String

Global ECluster_AllocateMemory._ECluster_AllocateMemory
Global ECluster_FreeMemory._ECluster_FreeMemory

; Buffer fr String - bergabe

Macro ECluster_Master_GetSlaveJobInfos(ECluster_Master, Slave, ForceRefresh = 0)
  PeekS(_ECluster_Master_GetSlaveJobInfos(ECluster_Master, Slave, ForceRefresh))
EndMacro

Macro ECluster_Master_Event_JobString(ECluster_Master)
  PeekS(_ECluster_Master_Event_JobString(ECluster_Master))
EndMacro

Macro ECluster_Parameter_GetString(ParameterStack)
  PeekS(_ECluster_Parameter_GetString(ParameterStack))
EndMacro

Macro ECluster_Slave_Event_Job(ECluster_Slave)
  PeekS(_ECluster_Slave_Event_Job(ECluster_Slave))
EndMacro

; Initialisieren
Procedure ECluster_Init(Dll$ = "ECluster.dll")
  Protected ECluster_Dll
  ECluster_Dll = OpenLibrary(#PB_Any, DLL$)
  If ECluster_DLL
    ECluster_Slave_Init = GetFunction(ECluster_Dll, "ECluster_Slave_Init")
    ECluster_Slave_DeInit = GetFunction(ECluster_Dll, "ECluster_Slave_DeInit")
    ECluster_Slave_Pause = GetFunction(ECluster_Dll, "ECluster_Slave_Pause")
    ECluster_Slave_Resume = GetFunction(ECluster_Dll, "ECluster_Slave_Resume")
    ECluster_Slave_CountMasters = GetFunction(ECluster_Dll, "ECluster_Slave_CountMasters")
    ECluster_Slave_IterateMasters = GetFunction(ECluster_Dll, "ECluster_Slave_IterateMasters")
    ECluster_Slave_KickMaster = GetFunction(ECluster_Dll, "ECluster_Slave_KickMaster")
    ECluster_Slave_SetMasterJobLimit = GetFunction(ECluster_Dll, "ECluster_Slave_SetMasterJobLimit")
    ECluster_Slave_CountMasterJobs = GetFunction(ECluster_Dll, "ECluster_Slave_CountMasterJobs")
    ECluster_Slave_GetMasterIP = GetFunction(ECluster_Dll, "ECluster_Slave_GetMasterIP")
    ECluster_Slave_GetMasterPort = GetFunction(ECluster_Dll, "ECluster_Slave_GetMasterPort")
    ECluster_Slave_GetMasterXtraData = GetFunction(ECluster_Dll, "ECluster_Slave_GetMasterXtraData")
    ECluster_Slave_SetMasterXtraData = GetFunction(ECluster_Dll, "ECluster_Slave_SetMasterXtraData")
    ECluster_Slave_SetMasterJobAccessCheck = GetFunction(ECluster_Dll, "ECluster_Slave_SetMasterJobAccessCheck")
    ECluster_Slave_CountJobs = GetFunction(ECluster_Dll, "ECluster_Slave_CountJobs")
    ECluster_Slave_SetReceiveBlockSize = GetFunction(ECluster_Dll, "ECluster_Slave_SetReceiveBlockSize")
    ECluster_Slave_SetCompressionOptions = GetFunction(ECluster_Dll, "ECluster_Slave_SetCompressionOptions")
    ECluster_Slave_SetCallback = GetFunction(ECluster_Dll, "ECluster_Slave_SetCallback")
    ECluster_Slave_ThreadReceive = GetFunction(ECluster_Dll, "ECluster_Slave_ThreadReceive")
    ECluster_Slave_AddJob = GetFunction(ECluster_Dll, "ECluster_Slave_AddJob")
    ECluster_Slave_LogEvents = GetFunction(ECluster_Dll, "ECluster_Slave_LogEvents")
    ECluster_Slave_Event = GetFunction(ECluster_Dll, "ECluster_Slave_Event")
    ECluster_Slave_Event_Type = GetFunction(ECluster_Dll, "ECluster_Slave_Event_Type")
    ECluster_Slave_Event_Master = GetFunction(ECluster_Dll, "ECluster_Slave_Event_Master")
    _ECluster_Slave_Event_Job = GetFunction(ECluster_Dll, "ECluster_Slave_Event_Job")
    ECluster_Slave_Job_SendMessage = GetFunction(ECluster_Dll, "ECluster_Slave_Job_SendMessage")
    
    ECluster_Master_Init = GetFunction(ECluster_Dll, "ECluster_Master_Init")
    ECluster_Master_DeInit = GetFunction(ECluster_Dll, "ECluster_Master_DeInit")
    ECluster_Master_SetReceiveBlockSize = GetFunction(ECluster_Dll, "ECluster_Master_SetReceiveBlockSize")
    ECluster_Master_SetCompressionOptions = GetFunction(ECluster_Dll, "ECluster_Master_SetCompressionOptions")
    ECluster_Master_ThreadReceive = GetFunction(ECluster_Dll, "ECluster_Master_ThreadReceive")
    ECluster_Master_SetCallback = GetFunction(ECluster_Dll, "ECluster_Master_SetCallback")
    ECluster_Master_AddSlave = GetFunction(ECluster_Dll, "ECluster_Master_AddSlave")
    ECluster_Master_RemoveSlave = GetFunction(ECluster_Dll, "ECluster_Master_RemoveSlave")
    ECluster_Master_CountSlaves = GetFunction(ECluster_Dll, "ECluster_Master_CountSlaves")
    ECluster_Master_IterateSlaves = GetFunction(ECluster_Dll, "ECluster_Master_IterateSlaves")
    ECluster_Master_GetSlaveIP = GetFunction(ECluster_Dll, "ECluster_Master_GetSlaveIP")
    ECluster_Master_GetSlaveXtraData = GetFunction(ECluster_Dll, "ECluster_Master_GetSlaveXtraData")
    ECluster_Master_SetSlaveXtraData = GetFunction(ECluster_Dll, "ECluster_Master_SetSlaveXtraData")
    ECluster_Master_PingSlave = GetFunction(ECluster_Dll, "ECluster_Master_PingSlave")
    _ECluster_Master_GetSlaveJobInfos = GetFunction(ECluster_Dll, "ECluster_Master_GetSlaveJobInfos")
    ECluster_Master_CountSlaveJobs = GetFunction(ECluster_Dll, "ECluster_Master_CountSlaveJobs")
    ECluster_Master_CountSlaveWorkers = GetFunction(ECluster_Dll, "ECluster_Master_CountSlaveWorkers")
    ECluster_Master_DoJob = GetFunction(ECluster_Dll, "ECluster_Master_DoJob")
    
    ECluster_Master_WaitEvent = GetFunction(ECluster_Dll, "ECluster_Master_WaitEvent")
    ECluster_Master_Event = GetFunction(ECluster_Dll, "ECluster_Master_Event")
    ECluster_Master_Event_Type = GetFunction(ECluster_Dll, "ECluster_Master_Event_Type")
    ECluster_Master_Event_Slave = GetFunction(ECluster_Dll, "ECluster_Master_Event_Slave")
    ECluster_Master_Event_JobId = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobId")
    ECluster_Master_Event_JobMem = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobMem")
    ECluster_Master_Event_JobMemSize = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobMemSize")
    ECluster_Master_Event_JobLong = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobLong")
    ECluster_Master_Event_JobInteger = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobInteger")
    ECluster_Master_Event_JobFloat = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobFloat")
    ECluster_Master_Event_JobDouble = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobDouble")
    ECluster_Master_Event_JobQuad = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobQuad")
    _ECluster_Master_Event_JobString = GetFunction(ECluster_Dll, "ECluster_Master_Event_JobString")
    
    ECluster_Parameter_New = GetFunction(ECluster_Dll, "ECluster_Parameter_New")
    ECluster_Parameter_AddMem = GetFunction(ECluster_Dll, "ECluster_Parameter_AddMem")
    ECluster_Parameter_AddLong = GetFunction(ECluster_Dll, "ECluster_Parameter_AddLong")
    ECluster_Parameter_AddInteger = GetFunction(ECluster_Dll, "ECluster_Parameter_AddInteger")
    ECluster_Parameter_AddFloat = GetFunction(ECluster_Dll, "ECluster_Parameter_AddFloat")
    ECluster_Parameter_AddDouble = GetFunction(ECluster_Dll, "ECluster_Parameter_AddDouble")
    ECluster_Parameter_AddQuad = GetFunction(ECluster_Dll, "ECluster_Parameter_AddQuad")
    ECluster_Parameter_AddString = GetFunction(ECluster_Dll, "ECluster_Parameter_AddString")
    ECluster_Parameter_GetMem = GetFunction(ECluster_Dll, "ECluster_Parameter_GetMem")
    ECluster_Parameter_GetLong = GetFunction(ECluster_Dll, "ECluster_Parameter_GetLong")
    ECluster_Parameter_GetInteger = GetFunction(ECluster_Dll, "ECluster_Parameter_GetInteger")
    ECluster_Parameter_GetFloat = GetFunction(ECluster_Dll, "ECluster_Parameter_GetFloat")
    ECluster_Parameter_GetDouble = GetFunction(ECluster_Dll, "ECluster_Parameter_GetDouble")
    ECluster_Parameter_GetQuad = GetFunction(ECluster_Dll, "ECluster_Parameter_GetQuad")
    _ECluster_Parameter_GetString = GetFunction(ECluster_Dll, "ECluster_Parameter_GetString")
    
    ECluster_Return_Memory = GetFunction(ECluster_Dll, "ECluster_Return_Memory")
    ECluster_Return_Long = GetFunction(ECluster_Dll, "ECluster_Return_Long")
    ECluster_Return_Integer = GetFunction(ECluster_Dll, "ECluster_Return_Integer")
    ECluster_Return_Float = GetFunction(ECluster_Dll, "ECluster_Return_Float")
    ECluster_Return_Double = GetFunction(ECluster_Dll, "ECluster_Return_Double")
    ECluster_Return_Quad = GetFunction(ECluster_Dll, "ECluster_Return_Quad")
    ECluster_Return_String = GetFunction(ECluster_Dll, "ECluster_Return_String")
    
    ECluster_AllocateMemory = GetFunction(ECluster_Dll, "ECluster_AllocateMemory")
    ECluster_FreeMemory = GetFunction(ECluster_Dll, "ECluster_FreeMemory")
    
    ProcedureReturn ECluster_Dll
  EndIf
EndProcedure

DisableExplicit

; IDE Options = PureBasic 4.31 Beta 2 (Windows - x86)
; CursorPosition = 1145
; FirstLine = 1116
; Folding = -
; EnableXP
; DisableDebugger